#!/usr/bin/python
# -*- coding: utf-8 -*-

import xml.dom.minidom as minidom
import pprint,re,os,sys

def parse_ts(ts_fname):
	doc = minidom.parse(open(ts_fname))
	tr_dict = {}
	ts_elem = doc.getElementsByTagName('TS')[0]
	lang = ts_elem.getAttribute('language')
	for ctx_elem in doc.getElementsByTagName('context'):
		name = ctx_elem.getElementsByTagName('name')[0]
		context = name.childNodes[0].data
		tr_dict[context] = {}
		for msg_elem in ctx_elem.getElementsByTagName('message'):
			src_elem = msg_elem.getElementsByTagName('source')
			if not len(src_elem):
				continue
			source = src_elem[0].childNodes[0].data
			if source not in tr_dict[context]:
				tr_dict[context][source] = {}

			comment = u''
			c_elem = msg_elem.getElementsByTagName('comment')
			if len(c_elem):
				comment = c_elem[0].childNodes[0].data
			if comment not in tr_dict[context][source]:
				tr_dict[context][source][comment] = {}
			
			translation = None
			tr_elem = msg_elem.getElementsByTagName('translation')
			if len(tr_elem):
				if len(tr_elem[0].childNodes):
					translation = tr_elem[0].childNodes[0].data
				tr_dict[context][source][comment]['type'] = None
				if tr_elem[0].hasAttribute('type'):
					tr_dict[context][source][comment]['type'] = tr_elem[0].getAttribute('type')
			tr_dict[context][source][comment]['translation'] = translation
			
			filename = None
			line = None
			l_elem = msg_elem.getElementsByTagName('location')
			if len(l_elem):
				filename = l_elem[0].getAttribute('filename')
				line = l_elem[0].getAttribute('line')
			tr_dict[context][source][comment]['filename'] = filename
			tr_dict[context][source][comment]['line'] = line
	return lang,tr_dict


def lrelease(ts_fname):
	lang,tr_dict = parse_ts(ts_fname)
	basedir = os.path.dirname(ts_fname)
	if not os.path.exists(basedir):
		os.makedirs(basedir)
	for context,sources in tr_dict.items():
		translations = {}
		for source,comments in sources.items():
			translations[source] = {}
			for comment,trinfo in comments.items():
				if trinfo['translation']:
					translations[source][comment] = trinfo['translation']
			if not len(translations[source]):
				del translations[source]
		f = open('%s/%s.py' % (basedir,context),'w')
		f.write('''# -*- coding: utf-8 -*-

translations = ''')
		pprint.pprint(translations,f)
		f.close()

if __name__=='__main__':
	if len(sys.argv)<2:
		sys.stderr.write("usage: %s <ts-file>\n" % sys.argv[0])
		sys.stderr.write("Example: %s admin_da.ts\n" % sys.argv[0])
		sys.exit(1)
	lrelease(sys.argv[1])





import logging,logging.handlers
import traceback,imp
from StringIO import StringIO
from os.path import join


def get_traceback():
	strio = StringIO()
	traceback.print_exc(file=strio)
	return strio.getvalue()

pysite_logger = None

def logger(conf):
	global pysite_logger
	if not pysite_logger:
		log_filename = conf.logfile
		pysite_logger = logging.getLogger(conf.sitename)
		pysite_logger.setLevel(logging.DEBUG)

		# Add the log message handler to the logger
		if len(pysite_logger.handlers)==0:
			handler = logging.handlers.RotatingFileHandler(log_filename, maxBytes=4000000, backupCount=5)
			formatter = logging.Formatter("%(asctime)s - %(message)s")
			handler.setFormatter(formatter)
			pysite_logger.addHandler(handler)
	return pysite_logger


from pysite.exceptions.conf import *
import imp

class PySiteConfiguration(object):
	def __init__(self,sitename=None,sitetitle=None,logfile=None):
		if sitename:
			self.sitename = sitename
			
		if not getattr(self,'sitename',None):
			raise ConfException('Mandatory configuration member "sitename" is missing')
		
		if sitetitle:
			self.sitetitle = sitetitle
		elif not getattr(self,'sitetitle',None):
			self.sitetitle = self.sitename

		if logfile:
			self.logfile = logfile
		elif not getattr(self,'logfile',None):
			self.logfile = '%s.log' % self.sitename

global_conf = None

def getConfiguration(conffile=None):
	global global_conf
	if not global_conf:
		confmod = imp.load_source('conf',conffile)
		global_conf = confmod.siteconf()
	return global_conf


# -*- coding: utf-8 -*-
from jinja2 import Template
from os.path import dirname,normpath,join,exists,abspath
import re,cgi,sys,imp
from wsgiref.util import request_uri
from pysite.tools.log import logger,get_traceback
from pysite.localization import localization
from pysite.conf import getConfiguration
import httpheader

rx_login_id = re.compile('mv_login_id=([-_\.a-zA-Z0-9]+)')

def redirect(location,cookies,start_response):
	response_headers = [
		('Content-type', 'text/html; charset=utf-8'),
		('Location',location),
		('Content-Length','0')]
	for c in cookies:
		response_headers += [('Set-Cookie', c)]
	start_response('302 Redirect', response_headers)
	return ['']

# Probe function
def probe_attribute(query_string,regexp):
	miter = regexp.finditer(query_string)
	try:
		match = miter.next()
		return match.groups()[0]
	except StopIteration:
		return None

class PySiteApplication(object):

	def __init__(self,basedir):
		self.basedir = abspath(basedir)
		self.conf = getConfiguration(join(basedir,'conf.py'))
		self.templates_dir = join(basedir,'templates')
		self.subhandlers_dir = join(basedir,'subhandlers')
		self.translations_dir = join(basedir,'translations')
		sys.path.append(basedir)
	
	def __call__(self,environ, start_response):
		global rx_login_id,mv_login_version
		
		log = logger(self.conf)
		status = "200 OK"
		output = ""

		mv_login_id = probe_attribute(environ['QUERY_STRING'],rx_login_id)
		uri_parts = request_uri(environ).split('?')
		if mv_login_id:
			yield redirect(
				uri_parts[0],
				[],
				start_response)

		locales = localization(self.translations_dir)
		ls = locales.lang_support()
		lang = None # Default
		default_lang = 'en'
		browser_langs = []
		if 'HTTP_ACCEPT_LANGUAGE' in environ:
			browser_langs = httpheader.parse_accept_language_header(environ['HTTP_ACCEPT_LANGUAGE'])
		for l in browser_langs:
			if lang:
				break
			for lp in l[0].parts:
				if lp in ls:
					lang = lp
					break
		if not lang:
			lang = default_lang
		mv_login_id = None
		if 'HTTP_COOKIE' in environ:
			http_cookies = environ['HTTP_COOKIE']
			mv_login_id = probe_attribute(http_cookies,rx_login_id)

		template = environ['PATH_INFO'][1:]
		if template=='':
			template = 'main'
		template_info = {'tr':{},'tr_common':{}}
		template_info['supported_countries'] = [
			{'short': 'dk', 'title': u'Danmark'},
			{'short': 'se', 'title': u'Sverige'},
			{'short': 'no', 'title': u'Norge'}
		]
		template_info.update(locales.tr(lang,'common',disambiguation='school'))
		template_info.update(locales.tr(lang,template,disambiguation='school'))
		template_info['tr_common'].update(locales.tr(lang,'common',disambiguation='school'))
		template_info['tr'].update(locales.tr(lang,template,disambiguation='school'))
		
		# Request translator
		def translate(source,context=template,comment=None,lang=lang,locales=locales):
			return locales.tr(lang,context=context,source=source,disambiguation=comment)

		subhandler = None
		try:
			subhandler = imp.load_source('subhandlers.%s' % template, join(self.basedir,'subhandlers','%s.py' % template))
		except:
			log.warning(get_traceback())
			pass
		
		response_headers = []

		environ['lang_support'] = locales.lang_support
		environ['tr'] = locales.tr
		environ['logger'] = log
		environ['lang'] = lang
		init = getattr(subhandler,'init',None)
		if init:
			try:
				init(template_info,response_headers,environ,translate)
			except Exception as e:
				log.warning(get_traceback())
				raise e
		
		subhandler_redirect = getattr(subhandler,'redirect',None)
		if subhandler_redirect:
			cookies = []
			location = subhandler_redirect(template_info,cookies,environ)
			if location:
				yield redirect(location,cookies,start_response)
		
		template_info['translate'] = translate
		template = join(self.templates_dir,'%s.jinja' % template)
		if exists(template):
			jinja_temp = Template(unicode(open(template).read(),'utf-8'))
			output = jinja_temp.render(template_info).encode('utf-8')
		else:
			output = 'Template: "%s" does not exist' % template 
		
		response_headers += [
			('Content-type', 'text/html; charset=utf-8'),
			('Content-Length', str(len(output)))]
		
		start_response(status, response_headers)
		yield output




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import sys,os

if sys.version_info[0]==2:
	other_version = 3
else:
	other_version = 2


readme = changes = ''
if os.path.exists('README.rst'):
	readme = open('README.rst').read()
	changes = open('CHANGES.rst').read()

VERSION = '0.1.2'

SHORT_DESC = "Create simple yet powerful WSGI based sites, utilizing Jinja2 and Qt's TS-file format for localization"
PACKAGES = find_packages('src')

setup(
    name='pysite',
    packages=PACKAGES,
    package_dir={'':'src'},
    version=VERSION,
    description=SHORT_DESC,
    long_description='\n\n'.join([readme, changes]),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',       
    ],
    keywords= ['wsgi', 'website', 'www', 'website', 'framework'],
    author='Jakob Simon-Gaarde',
    author_email='jakob@simon-gaarde.dk',
    maintainer = 'Jakob Simon-Gaarde',
    maintainer_email = 'jakob@simon-gaarde.dk',
    install_requires=['jinja2'],
    requires=['jinja2'],
    provides=['pysite'],
    license='LGPL3',
    scripts=['scripts/pysite'],
    zip_safe=False
)
